-- ######################
-------------------------
-- Enchant Woot-O-Matic
--
-- Written by August Lilleaas (Gramr/leethal)
-- Do what you want to with this code (even launch nukes!)
--
-------------------------
-- ######################

-- ######################
-- Initializing the EnchantWootOMatic object
-- ######################

local EnchantWootOMatic = {}
local frames = {GameTooltip, ItemRefTooltip}
local L = EnchantWootOMaticLocals

-- ######################
-- Zeh DB
-- ######################

local enchant_list = {}

for k,v in pairs(EnchantWootOMaticLocals.procs) do
    enchant_list[k] = string.format("%s %s", ITEM_SPELL_TRIGGER_ONPROC, v)
end
for k,v in pairs(EnchantWootOMaticLocals.equips) do
    enchant_list[k] = string.format("%s %s", ITEM_SPELL_TRIGGER_ONEQUIP, v)
end
    
    
-- ######################
-- The Stuff
-- ######################

function EnchantWootOMatic:Enable()
    for _, v in pairs(frames) do
        local old = v:GetScript("OnShow")
        v:SetScript("OnShow", function(...) 
          self:DoTheWoot(v)
          if old then old(...) end
        end)
    end
end

function EnchantWootOMatic:InitSettings()
	if (not EnchantWootOMaticDB) then
		EnchantWootOMaticDB = {
			color = {
				r = 0,
				g = 1,
				b = 0,
			}
		}
	end
	
	local LibSimpleOptions = LibStub("LibSimpleOptions-1.0")
	LibSimpleOptions.AddSlashCommand('Enchant Woot-O-Matic', '/ewom')
	LibSimpleOptions.AddOptionsPanel('Enchant Woot-O-Matic', function(panel)
		local title, subtext = panel:MakeTitleTextAndSubText('Enchant Woot-O-Matic', 'Totally super cool configuration panel.')
	
		local picker = panel:MakeColorPicker(
			'name', 'Text color',
			'description', 'Set the color of the enchant description tooltip text',
			'hasAlpha', false,
			'defaultR', EnchantWootOMaticDB.color.r,
			'defaultG', EnchantWootOMaticDB.color.g,
			'defaultB', EnchantWootOMaticDB.color.b,
			'getFunc', function()
				return EnchantWootOMaticDB.color.r, EnchantWootOMaticDB.color.g, EnchantWootOMaticDB.color.b
			end,
			'setFunc', function(r, g, b)
				EnchantWootOMaticDB.color.r = r
				EnchantWootOMaticDB.color.g = g
				EnchantWootOMaticDB.color.b = b
			end
		)
		picker:SetPoint('TOPLEFT', subtext, 'BOTTOMLEFT', 0, -16)
		
		local seedButton = panel:MakeButton(
			'name', 'Create test tooltip',
			'description', 'Insert an item link into the chat with a dummy tooltip.',
			'func', function()
				DEFAULT_CHAT_FRAME:AddMessage("Dummy tooltip: \124cff9d9d9d\124Hitem:20030:1900:0:0:0:0:0:0:0\124h[Pet Rock]\124h\124r");
			end
		)
		seedButton:SetPoint('TOPLEFT', picker, 'BOTTOMLEFT', 0, -16)
	end)
end

function EnchantWootOMatic:RGBToHex()
	local color = EnchantWootOMaticDB.color
	return string.format("%02x%02x%02x", color.r * 255, color.g * 255, color.b * 255)
end

function EnchantWootOMatic:DoTheWoot(frame)
    local itemlink =  self:GetItemLinkFromFrame(frame)
    if itemlink then -- is this an item?
        self:GenerateTooltip(frame, itemlink)
    end
end

function EnchantWootOMatic:GenerateTooltip(frame, itemlink)
    local enchant_id = self:GetEnchantID(itemlink)
    if enchant_id and tonumber(enchant_id) > 0 then
        self:IterateTooltip(frame)
    end
end

function EnchantWootOMatic:IterateTooltip(frame)
    local width = frame:GetWidth()
    for i=1, frame:NumLines() do
        local tooltip_row = getglobal(frame:GetName().."TextLeft"..i)
        local text = tooltip_row:GetText()
        for k,v in pairs(enchant_list) do
            if text == k then
                tooltip_row:SetWidth(width)
                tooltip_row:SetText(string.format("%s\n|CFF" .. self:RGBToHex() .. "%s|r", k, v), nil, nil, nil,0.5, 1)
            end
        end
    end
    frame:Show()
end

function EnchantWootOMatic:GetItemLinkFromFrame(frame)
    return select(2, frame:GetItem())
end

function EnchantWootOMatic:GetEnchantID(itemstring)
    if itemstring then
        local _, _, _, enchant_id, _, _, _, _, _, _ = string.find(itemstring, "item:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%-?%d+):(%-?%d+)")
        return enchant_id
    end
end


local eventFrame = CreateFrame('Frame')
eventFrame:RegisterEvent('PLAYER_LOGIN')
eventFrame:SetScript('OnEvent', function()
	EnchantWootOMatic:InitSettings()
	EnchantWootOMatic:Enable()
end)